@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
 # contact mailto:thenault@gmail.com
@@ -24,8 +24,11 @@ inference utils.
 
 __docformat__ = "restructuredtext en"
 
+from contextlib import contextmanager
 
 from logilab.common.compat import builtins
+
+from logilab.astng import BUILTINS_MODULE
 from logilab.astng.exceptions import InferenceError, ASTNGError, \
                                        NotFoundError, UnresolvableName
 from logilab.astng.as_string import as_string
@@ -78,6 +81,12 @@ class InferenceContext(object):
         clone.boundnode = self.boundnode
         return clone
 
+    @contextmanager
+    def restore_path(self):
+        path = set(self.path)
+        yield
+        self.path = path
+
 def copy_context(context):
     if context is not None:
         return context.clone()
@@ -242,7 +251,7 @@ class UnboundMethod(Proxy):
         # If we're unbound method __new__ of builtin object, the result is an
         # instance of the class given as first argument.
         if (self._proxied.name == '__new__' and
-                self._proxied.parent.frame().qname() == '__builtin__.object'):
+                self._proxied.parent.frame().qname() == '%s.object' % BUILTINS_MODULE):
             return (x is YES and x or Instance(x) for x in caller.args[0].infer())
         return self._proxied.infer_call_result(caller, context)
 
@@ -268,7 +277,7 @@ class Generator(Instance):
         return True
 
     def pytype(self):
-        return '__builtin__.generator'
+        return '%s.generator' % BUILTINS_MODULE
 
     def display_type(self):
         return 'Generator'
@@ -336,6 +345,7 @@ class NodeNG(object):
     lineno = None
     fromlineno = None
     tolineno = None
+    col_offset = None
     # parent node in the tree
     parent = None
     # attributes containing child node(s) redefined in most concrete classes:
